// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "timer.hpp"

namespace dxp {
namespace sys {

timer appTimer;

timer::timer()
{
   #ifdef dLinux
		reset();
   #endif

   #ifdef dWindows
      if(!QueryPerformanceFrequency((LARGE_INTEGER*)&rate) || !rate)
      {
         active=false;
         return;
      }
      rateinv=1.0/(double)rate;
      active=true;
   #endif
}

void timer::reset()
{
   #ifdef dWindows
      if(active)
			QueryPerformanceCounter((LARGE_INTEGER*)&start);
   #endif

   #ifdef dLinux
      struct timeval st;
      gettimeofday(&st, NULL);
		start=(st.tv_sec*1000.0 + st.tv_usec/1000.0);
   #endif
}

double timer::get()
{
   #ifdef dWindows
      if (!active) return -1;
      __int64 end;
      QueryPerformanceCounter((LARGE_INTEGER*)&end);
      return (double)(end-start)*rateinv;
   #endif
   
   #ifdef dLinux
      struct timeval now;
      gettimeofday(&now, NULL);
      return 
      (
      	(
now.tv_sec*1000.0 + now.tv_usec/1000.0) - start ) / 1000.0
;
   #endif
}

} // sys
} // dxp
