// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

// Authors:
// - Rafael Jannone: Windows code, original interface
// - Daniel Basso: Linux code, new interface

#ifndef hTimer
#define hTimer

#define dLinux

#ifdef dWindows
   #include <windows.h>
   #define dSleep(x) Sleep(x)
#endif
#ifdef dLinux
   #include <unistd.h>
   #include <sys/time.h>
   #define dSleep(x) usleep(x*1000)
#endif

namespace dxp {
namespace sys {

class timer
{
   public:
      timer();
      void reset();
      double get();
      operator double() { return get(); }

	private:
      #ifdef dWindows
         __int64 start;
         __int64 rate;
         double  rateinv;
         bool    active;
      #endif
      #ifdef dLinux
         double start;
      #endif
};

extern timer appTimer;

class counter
{
	public:
		counter() { reset(); }
		void reset() { count=0; startTime=appTimer.get(); }
		void add(double value) { count+=value; }
		void adjust(double timeWindow) 
		{ 
			count=getRate()*timeWindow;
			startTime=appTimer.get()-timeWindow;
		}
		double getCount() { return count; }
		double getTime() { return appTimer.get()-startTime; }
		double getRate() { return count/getTime(); }
		double estimateFinish(double destCount) 
		{ 
			return (destCount-count)/getRate();
		}
		
		counter& operator ++ (int) { add(1.0); return *this; }

	protected:
		double count;
		double startTime;
};


} // sys
} // dxp

#endif //hTimer
